
// create fonts
surface.CreateFont( "Veranda", 12, 200, true, false, "Silk" );

// get the materials we're going to use
local matGradient = Material( "gui/gradient" );


/*------------------------------------
	called when the skin is applied
------------------------------------*/
function SKIN:Apply( panel )

	// change the control buttons
	MuseX.CallSkinHook( "ControlsPlay", "musex/skins/Silk/play" );
	MuseX.CallSkinHook( "ControlsNext", "musex/skins/Silk/next" );
	MuseX.CallSkinHook( "ControlsPrevious", "musex/skins/Silk/previous" );
	MuseX.CallSkinHook( "ControlsStop", "musex/skins/Silk/stop" );
	MuseX.CallSkinHook( "ControlsMute", "musex/skins/Silk/mute" );
	MuseX.CallSkinHook( "ControlsRepeat", "musex/skins/Silk/repeat" );
	MuseX.CallSkinHook( "ControlsShuffle", "musex/skins/Silk/shuffle" );
	
	// change the playlist buttons
	MuseX.CallSkinHook( "PlaylistAdd", "musex/skins/Silk/add" );
	MuseX.CallSkinHook( "PlaylistRemove", "musex/skins/Silk/remove" );
	
	// change the slider knob
	MuseX.CallSkinHook( "SliderKnob", "musex/skins/Silk/slider" );

end


/*------------------------------------
	paint the main frame
------------------------------------*/
function SKIN:PaintMXMainFrame( panel )

	// get width and height
	local w, h = panel:GetSize();
	
	// draw background
	draw.RoundedBox( 4, 1, 1, w - 2, h - 2, Color( 230, 230, 230, 255 ) );
	
	// draw the gradient overlay
	surface.SetMaterial( matGradient );
	surface.SetDrawColor( 0, 0, 0, 100 )
	surface.DrawTexturedRectRotated( w * 0.5, h * 0.5, w, h, 180 )
	
	// draw title bar
	draw.RoundedBox( 4, 2, 2, w - 4, 20, Color( 82, 131, 192, 200 ) );

end


/*------------------------------------
	scheme the main frame
------------------------------------*/
function SKIN:SchemeMXFrame( panel )

	// change the title font & font color
	panel.lblTitle:SetTextColor( Color( 0, 0, 0, 255 ) );
	panel.lblTitle:SetFont( "Silk" );
	
	// set the system button colors
	panel.btnClose:SetTextColor( Color( 0, 0, 0, 255 ) );
	if ( panel.Minimize ) then panel.Minimize:SetTextColor( Color( 0, 0, 0, 255 ) ); end
	if ( panel.ViewMode ) then panel.ViewMode:SetTextColor( Color( 0, 0, 0, 255 ) ); end
	
end


/*------------------------------------
	paint the slider
------------------------------------*/
function SKIN:PaintMXSlider( panel )

	// get width and height
	local w, h = panel:GetSize();
	
	// draw slider channel
	draw.RoundedBox( 4, 4, 5, w - 9, h - 10, Color( 20, 20, 150, 200 ) );
	draw.RoundedBox( 4, 4, 6, w - 9, h - 12, Color( 82, 131, 192, 200 ) );
	
end



/*------------------------------------
	paint the listview
------------------------------------*/
function SKIN:PaintMXListView( panel )

	// get width and height
	local w, h = panel:GetSize();
	
	// draw gradient overlay
	surface.SetMaterial( matGradient );
	surface.SetDrawColor( 0, 0, 0, 60 )
	surface.DrawTexturedRectRotated( w * 0.5, h * 0.5, w, h, 180 )
	
	// shrink width & height
	w = w - 1;
	h = h - 1;
	
	// draw blue outline
	surface.SetDrawColor( 20, 20, 150, 200 );
	surface.DrawLine( 0, 0, w, 0 );
	surface.DrawLine( w, 0, w, h );
	surface.DrawLine( w, h, 0, h );
	surface.DrawLine( 0, h, 0, 0 );

end


/*------------------------------------
	paint the listview header
------------------------------------*/
function SKIN:PaintMXListViewHeader( panel )

	// get width and height
	local w, h = panel:GetSize();
	
		// check if background painting is enabled
		if ( panel.m_bBackground ) then
		
		
		// get width & height
		local w, h = panel:GetSize();
		
		// check if background is enabled
		if ( panel.m_bBackground ) then
		
			// set the default colors
			local col1 = Color( 20, 20, 150, 200 );
			local col2 = Color( 82, 131, 192, 255 );
			
			// grey if disabled
			if ( panel:GetDisabled() ) then
			
				col1 = Color( 100, 100, 100, 255 );
				
			// swap colors if pressed or selected
			elseif ( panel.Depressed || panel:GetSelected() ) then
			
				col1 = Color( 0, 0, 0, 255 );	
				col2 = Color( 0, 0, 0, 255 );
			
			// darken if hovered
			elseif ( panel.Hovered ) then
			
				col2 = Color( 62, 101, 152, 255 );
				
			end
			
			// draw the border
			draw.RoundedBox( 4, 0, 0, w, h, Color( col1.r, col1.g, col1.b, col1.a ) );
			
			// draw inner color
			draw.RoundedBox( 4, 1, 1, w - 2, h - 2, Color( col2.r, col2.g, col2.b, col2.a ) );
		
		end
	
	end

end


/*------------------------------------
	scheme the listview header
------------------------------------*/
function SKIN:SchemeMXListViewHeader( panel )

	// set the font
	panel:SetFont( "Silk" );

end


/*------------------------------------
	paint the listview line
------------------------------------*/
function SKIN:PaintMXListViewLine( panel )

	// check if this line is the current song
	if ( panel.Filename == MuseX.CurrentSong ) then
	
		// orange
		surface.SetDrawColor( 255, 200, 0, 220 );
		
		// get width and height
		local w, h = panel:GetSize();
		
		// shrink
		w = w - 1;
		h = h - 1;
		
		// draw outline
		surface.DrawLine( 0, 0, w, 0 );
		surface.DrawLine( w, 0, w, h );
		surface.DrawLine( w, h, 0, h );
		surface.DrawLine( 0, h, 0, 0 );
		
	end
	
	// color variable
	local col = nil;
	
	// blue if selected
	if ( panel:IsSelected() ) then
	
		col = Color( 255, 200, 0, 150 );
		
	// grey if hovered
	elseif ( panel.Hovered ) then
	
		col = Color( 255, 255, 255, 100 );
		
	// alternating color
	elseif ( panel.m_bAlt ) then
	
		col = Color( 82, 131, 192, 80 )
		
	else
	
		// nothing to draw
		return;
				
	end
	
	// draw the background
	surface.SetDrawColor( col.r, col.g, col.b, col.a );
	surface.DrawRect( 0, 0, panel:GetWide(), panel:GetTall() );
	
end


/*------------------------------------
	scheme the listview line
------------------------------------*/
function SKIN:SchemeMXListViewLine( panel )

	// white font color
	panel:SetTextColor( Color( 255, 255, 255, 255 ) );
	
	// set the font
	panel:SetFont( "Silk" );

end


/*------------------------------------
	paint button
------------------------------------*/
function SKIN:PaintMXButton( panel )

	// get width & height
	local w, h = panel:GetSize();
	
	// check if background is enabled
	if ( panel.m_bBackground ) then
	
		// set the default colors
		local col1 = Color( 20, 20, 150, 200 );
		local col2 = Color( 82, 131, 192, 255 );
		
		// grey if disabled
		if ( panel:GetDisabled() ) then
		
			col1 = Color( 100, 100, 100, 255 );
			
		// swap colors if pressed or selected
		elseif ( panel.Depressed || panel:GetSelected() ) then
		
			col1 = Color( 0, 0, 0, 255 );	
			col2 = Color( 0, 0, 0, 255 );
		
		// darken if hovered
		elseif ( panel.Hovered ) then
		
			col2 = Color( 62, 101, 152, 255 );
			
		end
		
		// draw the border
		draw.RoundedBox( 4, 0, 0, w, h, Color( col1.r, col1.g, col1.b, col1.a ) );
		
		// draw inner color
		draw.RoundedBox( 4, 1, 1, w - 2, h - 2, Color( col2.r, col2.g, col2.b, col2.a ) );
	
	end

end


/*------------------------------------
	paint over button
------------------------------------*/
function SKIN:PaintOverMXButton( panel )

	// badger, badger, bagder
	
end


/*------------------------------------
	scheme button
------------------------------------*/
function SKIN:SchemeMXButton( panel )

	// set the font
	panel:SetFont( "Silk" );

end


/*------------------------------------
	paint the controls area
------------------------------------*/
function SKIN:PaintMXControls( panel )

	// mushroom, MUSHROOM!
	
end


/*------------------------------------
	paint combobox
------------------------------------*/
function SKIN:PaintMXComboBox( panel )

	// get width and height
	local w, h = panel:GetSize();
	
	// draw gradient overlay
	surface.SetMaterial( matGradient );
	surface.SetDrawColor( 0, 0, 0, 60 )
	surface.DrawTexturedRectRotated( w * 0.5, h * 0.5, w, h, 180 )
	
	// shrink width & height
	w = w - 1;
	h = h - 1;
	
	// draw blue outline
	surface.SetDrawColor( 20, 20, 150, 200 );
	surface.DrawLine( 0, 0, w, 0 );
	surface.DrawLine( w, 0, w, h );
	surface.DrawLine( w, h, 0, h );
	surface.DrawLine( 0, h, 0, 0 );
	
end


/*------------------------------------
	paint combobox item
------------------------------------*/
function SKIN:PaintMXComboBoxItem( panel )

	// check if selected
	if ( panel:GetSelected() ) then
	
		// draw blue background
		surface.SetDrawColor( 255, 200, 0, 150 );
		panel:DrawFilledRect();
		
	end

end


/*------------------------------------
	scheme combobox item
------------------------------------*/
function SKIN:SchemeMXComboBoxItem( panel )

	// white font color
	panel:SetTextColor( Color( 0, 0, 0, 255 ) );
	
	// set the font
	panel:SetFont( "Silk" );
	
end


/*------------------------------------
	paint text entry
------------------------------------*/
function SKIN:PaintMXTextEntry( panel )

	// draw the default text entry shit
	panel:DrawTextEntryText( Color( 0, 0, 0, 255), Color( 255, 200, 0, 255 ), Color( 255, 200, 0, 255 ) );
	
	// check if border is enabled
	if ( panel.m_bBorder ) then
	
		// draw border
		surface.SetDrawColor( 0, 0, 0, 255 );
		surface.DrawOutlinedRect( 0, 0, panel:GetWide(), panel:GetTall() );
	
	end
	
end


/*------------------------------------
	scheme text entry
------------------------------------*/
function SKIN:SchemeMXTextEntry( panel )

	// change text
	panel:SetFont( "Silk" );

end


/*------------------------------------
	scheme label
------------------------------------*/
function SKIN:SchemeMXLabel( panel )

	// black font color
	panel:SetTextColor( Color( 0, 0, 0, 255 ) );
	
	// set the font
	panel:SetFont( "Silk" );

end


/*------------------------------------
	paint the compact frame
------------------------------------*/
function SKIN:PaintMXCompactFrame( panel )

	// get width and height
	local w, h = panel:GetSize();
	
	// draw background
	draw.RoundedBox( 4, 1, 1, w - 2, h - 2, Color( 230, 230, 230, 255 ) );
	
	// draw the gradient overlay
	surface.SetMaterial( matGradient );
	surface.SetDrawColor( 0, 0, 0, 100 )
	surface.DrawTexturedRectRotated( w * 0.5, h * 0.5, w, h, 180 )
	
	// draw title bar
	draw.RoundedBox( 4, 2, 2, w - 4, 20, Color( 82, 131, 192, 200 ) );

end


/*------------------------------------
	scheme the compact frame
------------------------------------*/
function SKIN:SchemeMXCompactFrame( panel )

	// change the title font & font color
	panel.lblTitle:SetTextColor( Color( 0, 0, 0, 255 ) );
	panel.lblTitle:SetFont( "Silk" );
	
	// set the system button colors
	panel.btnClose:SetTextColor( Color( 0, 0, 0, 255 ) );
	panel.Minimize:SetTextColor( Color( 0, 0, 0, 255 ) );
	panel.ViewMode:SetTextColor( Color( 0, 0, 0, 255 ) );
	
end


// define
MuseX.DefineSkin( "Blue Silk", SKIN );
