
// panel object
local PANEL = {};


/*------------------------------------
	initialize main window
------------------------------------*/
function PANEL:Init( )

	// set the size
	self:SetSize( MuseX.GetSetting( "mainwindoww", MXSETTING_NUM ) or 575, MuseX.GetSetting( "mainwindowh", MXSETTING_NUM ) or 380 );

	// set tooltip
	self.btnClose:SetTooltip( "#musex_exit" );
	
	// resizing
	self.Resizing = false;
	self.ResizeInfo = { X = 0, Y = 0, StartWidth = 0, StartHeight = 0, Divider = 0 };
	
	// create minimize button
	self.Minimize = vgui.Create( "DSysButton", self );
	// set the icon
	self.Minimize:SetType( "down" );
	// dont draw default shit
	self.Minimize:SetDrawBorder( false );
	self.Minimize:SetDrawBackground( false );
	// set the tooltip
	self.Minimize:SetTooltip( "#musex_hide" );
	// setup the minimize command
	self.Minimize.DoClick = function ( button ) self:Hide(); end
	
	// create viewmode button
	self.ViewMode = vgui.Create( "DSysButton", self );
	// change the icon
	self.ViewMode:SetType( "grip" );
	// disable default border and background
	self.ViewMode:SetDrawBorder( false );
	self.ViewMode:SetDrawBackground( false );
	// set the tooltip
	self.ViewMode:SetTooltip( "#musex_switch_compact" );
	// setup the viewmode command
	self.ViewMode.DoClick = function ( button )
	
		// set as compact mode
		MuseX.SetSetting( "viewmode", 2 );
		
		// restart MuseX
		MuseX.Restart( true );
		
	end

	// build the window title
	self:SetTitle( ("MuseX version %s"):format( MuseX.GetVersion() ) );
	
	// setup the window
	self:SetDraggable( true );
	self:SetDeleteOnClose( false );
	
	// create the control object
	self.Controls = vgui.Create( "MXControls", self );
	
	// create the visualization area
	self.Visualization = vgui.Create( "MXVisualization" );
	
	// create the playlist
	self.Playlist = vgui.Create( "MXPlaylist" );
	
	// create the divider
	self.Divider = vgui.Create( "MXHorizontalDivider", self );
	// set the size of the divider grabbable area
	self.Divider:SetDividerWidth( 4 );
	// set the minimum sizes and starting sizes
	self.Divider:SetLeftMin( 250 );
	self.Divider:SetLeftWidth( MuseX.GetSetting( "maindivider", MXSETTING_NUM ) or 400 );
	self.Divider:SetRightMin( 100 );
	// setup the OnResized event
	self.Divider.OnResized = function( self ) MuseX.SetSetting( "maindivider", self:GetLeftWidth() ); end;
	
	// set the playlist and visualization inside the divider
	self.Divider:SetRight( self.Playlist );
	self.Divider:SetLeft( self.Visualization );
	
	// resizer
	self.Resizer = vgui.Create( "DSysButton", self );
	self.Resizer:SetType( "grip" );
	self.Resizer:SetDrawBorder( false );
	self.Resizer:SetDrawBackground( false );
	self.Resizer:SetCursor( "sizenwse" );
	self.Resizer.OnMousePressed = function( panel, mc )
	
		panel:MouseCapture( true );
		self.Resizing = true;
		self.ResizeInfo.X = gui.MouseX();
		self.ResizeInfo.Y = gui.MouseY();
		self.ResizeInfo.StartWidth = self:GetWide();
		self.ResizeInfo.StartHeight = self:GetTall();
		self.ResizeInfo.Divider = self.Divider:GetWide() - self.Divider:GetLeftWidth();
	
	end
	self.Resizer.OnMouseReleased = function( panel, mc )
	
		panel:MouseCapture( false );
		self.Resizing = false;
		
		// update position in settings
		MuseX.SetSetting( "mainwindoww", self:GetWide() );
		MuseX.SetSetting( "mainwindowh", self:GetTall() );
		MuseX.SetSetting( "maindivider", self.Divider:GetLeftWidth() );
	
	end
	
	// custom derma hooks
	Derma_Hook( self, "Paint", "Paint", "MXMainFrame" );
	Derma_Hook( self, "ApplySchemeSettings", "Scheme", "MXFrame" );
	self.AnimIn = Derma_Anim( "OpenAnim", self, self.OpenAnim );
	self.AnimOut = Derma_Anim( "CloseAnim", self, self.CloseAnim );

end


/*------------------------------------
	open the main window
------------------------------------*/
function PANEL:Open( )

	// ignore if we're already visible
	if ( self:IsVisible() ) then
	
		// poof
		return;
		
	end
	
	// close menus
	CloseDermaMenus();
	
	// clear flag
	self.IsHiding = false;
	
	// take focus and be visible
	self:MakePopup();
	self:SetVisible( true );
	
	// disable keyboard input
	self:SetKeyboardInputEnabled( false );
	
	// enable mouse input
	self:SetMouseInputEnabled( true );
	
	// halt the close animation
	self.AnimOut:Stop();
	
	// start the open animation
	self.AnimIn:Start( 0.1 );

end


/*------------------------------------
	handle the opening animation
------------------------------------*/
function PANEL:OpenAnim( anim, delta, data )

	// check if it has been started
	if ( anim.Started ) then
	
		// reset alpha
		self:SetAlpha( 0 );
		
	end
	
	// check if it has been finished
	if ( anim.Finished ) then
	
		// fully opaque
		self:SetAlpha( 255 );
		
		// all done
		return;
		
	end
	
	// animate the alpha
	self:SetAlpha( 255 * delta );

end


/*------------------------------------
	close the main window
------------------------------------*/
function PANEL:Close( skip, ishiding )
	
	// close menus
	CloseDermaMenus();
	
	// save flag
	self.IsHiding = ishiding;
	
	// halt the opening animation
	self.AnimIn:Stop();
	
	// check if we're skipping the animation
	if ( skip ) then
	
		// restore alpha
		self:SetAlpha( 255 );
		
		// not visible
		self:SetVisible( false );
	
	else
	
		// start the close animation
		self.AnimOut:Start( 0.1 );
		
	end

end


/*------------------------------------
	hide the main window
------------------------------------*/
function PANEL:Hide( )
	
	// pass to close function
	self:Close( true, true );

end


/*------------------------------------
	handle the closing animation
------------------------------------*/
function PANEL:CloseAnim( anim, delta, data )

	// check if the animation has started
	if ( anim.Started ) then
	
		// restore the alpha
		self:SetAlpha( 255 );
		
	end
	
	// check if the animation has finished
	if ( anim.Finished ) then
	
		// store alpha
		self:SetAlpha( 255 );
		
		// set as not visible
		self:SetVisible( false );
		
		// check if we're exiting
		if ( !self.IsHiding ) then
		
			// close MuseX
			MuseX.Close();
		
		end
		
		// 2girls1cup.com
		return;
		
	end
	
	// animate the alpha
	self:SetAlpha( 255 - 255 * delta );

end


/*------------------------------------
	layout the main window
------------------------------------*/
function PANEL:PerformLayout( )

	// let the BaseClass do its stuff
	self.BaseClass.PerformLayout( self );
	
	// size & position minimize button
	self.Minimize:SetPos( self:GetWide() - 40, 4 )
	self.Minimize:SetSize( 18, 18 );
	
	// size & position viewmode button
	self.ViewMode:SetPos( self:GetWide() - 58, 4 )
	self.ViewMode:SetSize( 18, 18 );
	
	// size & position the controls
	self.Controls:SetSize( self:GetWide(), 50 );
	self.Controls:SetPos( 0, self:GetTall() - self.Controls:GetTall() );
	
	// size and position the main divider area
	self.Divider:SetSize( self:GetWide() - 10, self:GetTall() - ( self.Controls:GetTall() + 5 + 30 ) );
	self.Divider:SetPos( 5, 30 );
	
	// resizer
	self.Resizer:SetSize( 16, 16 );
	self.Resizer:AlignBottom( 2 );
	self.Resizer:AlignRight( 2 );
	
end


/*------------------------------------
	main window think
------------------------------------*/
function PANEL:Think( )

	// call BaseClass
	self.BaseClass.Think( self );
	
	// check if they finishing moving the window
	if ( !self.Dragging && self.WasDragging ) then
	
		// update position in settings
		MuseX.SetSetting( "mainwindowx", self.X );
		MuseX.SetSetting( "mainwindowy", self.Y );
		
	end
	
	// resize
	if( self.Resizing ) then
	
		// calculate new height
		local newwidth = math.max( 575, self.ResizeInfo.StartWidth + ( gui.MouseX() - self.ResizeInfo.X ) );
		local newheight = math.max( 380, self.ResizeInfo.StartHeight + ( gui.MouseY() - self.ResizeInfo.Y ) );
		
		// set form size
		self:SetSize( newwidth, newheight );
		
		// increase vis size
		self.Divider:SetLeftWidth( newwidth - self.ResizeInfo.Divider );
	
	end
	
	// update previous dragging state
	self.WasDragging = self.Dragging;
	
	// run the animation functions
	self.AnimIn:Run();
	self.AnimOut:Run();

end


/*------------------------------------
	paint the main window
------------------------------------*/
function PANEL:Paint( )

	// BaseClass paint
	self.BaseClass.Paint( self );
	
	// all done!
	return true;
	
end


/* CUSTOM PANEL FUNCTIONS */


/*------------------------------------
	add a list of songs to playlist
------------------------------------*/
function PANEL:AddToPlaylist( songs )

	// loop through list of songs
	for _, song in pairs( songs ) do
	
		// add song
		self.Playlist:AddSong( song );
	
	end

end


/*------------------------------------
	remove a list of songs from
	the playlist
------------------------------------*/
function PANEL:RemoveFromPlaylist( songs )

	// loop through list of songs
	for _, song in pairs( songs ) do
	
		// remove song
		self.Playlist:RemoveSong( song );
	
	end

end


/*------------------------------------
	clear all songs from playlist
------------------------------------*/
function PANEL:ClearPlaylist( )

	// pass to playlist object
	self.Playlist:ClearPlaylist( );

end


/*------------------------------------
	get all songs on playlist
------------------------------------*/
function PANEL:GetPlaylist( )

	// pass to playlist
	return self.Playlist:GetPlaylist();

end


/*------------------------------------
	get the total count of songs
------------------------------------*/
function PANEL:GetPlaylistCount( )

	// pass to playlist
	return self.Playlist:GetPlaylistCount();

end


/*------------------------------------
	get the song a the specified
	playlist position
------------------------------------*/
function PANEL:GetPlaylistSong( num )

	// pass to playlist
	return self.Playlist:GetPlaylistSong( num );

end


/*------------------------------------
	get the currently highlighted
	playlist song
------------------------------------*/
function PANEL:GetSelectedPlaylistSong( )

	// pass to playlist
	return self.Playlist:GetSelectedPlaylistSong();
	
end


/*------------------------------------
	gets a songs position in list
------------------------------------*/
function PANEL:GetPlaylistPosition( filename )

	// pass to playlist
	return self.Playlist:GetPlaylistPosition( filename );
	
end


/*------------------------------------
	set the media info table
------------------------------------*/
function PANEL:SetMediaInfo( tbl )

	// pass to controls
	self.Controls:SetMediaInfo( tbl );

end


/*------------------------------------
	show playlist info
------------------------------------*/

function PANEL:ShowPlaylistInfo( text )

	// pass to controls
	self.Controls:ShowPlaylistInfo( text );

end


/*------------------------------------
	open visualization settings
------------------------------------*/
function PANEL:OpenVisualizationSettings( )

	// pass to visualization
	self.Visualization:OpenSettings();
	
end


/*------------------------------------
	refresh the volume bar position
------------------------------------*/
function PANEL:RefreshVolumePosition( )

	// pass to controls
	self.Controls:RefreshVolumePosition();

end


/*------------------------------------
	open visualization hotkey editor
------------------------------------*/
function PANEL:OpenHotKeyEditor( )

	// check if the hotkey editor has been opened
	if ( !self.HotKeyEditor || !self.HotKeyEditor:IsValid() ) then
	
		// if not, create it
		self.HotKeyEditor = vgui.Create( "MXHotKeyEditor" );
		
		// not visible
		self.HotKeyEditor:SetVisible( false );
		
		// set position
		self.HotKeyEditor:Center();
		
	end
	
	// open
	self.HotKeyEditor:Open();
	
end


/*------------------------------------
	set the position of the seek bar
------------------------------------*/
function PANEL:SetSeekPosition( frac )

	// pass to controls panel
	self.Controls:SetSeekPosition( frac );

end

// define
derma.DefineControl( "MXMain", "", PANEL, "DFrame" );
