
// panel object
local PANEL = {};


/*------------------------------------
	initialize the media info
------------------------------------*/
function PANEL:Init( )

	// create the container
	self.Container = vgui.Create( "MXMediaInfo_Container", self );

end


/*------------------------------------
	layout the media info
------------------------------------*/
function PANEL:PerformLayout( )

	// size & position the container
	self.Container:SetPos( 0, 0 );
	self.Container:SetSize( self:GetWide(), self:GetTall() );

end


/*------------------------------------
	paint the media info
------------------------------------*/
function PANEL:Paint( )

	// all done
	return true;

end


/* CUSTOM PANEL FUNCTIONS */


/*------------------------------------
	set the media info table
------------------------------------*/
function PANEL:SetMediaInfo( tbl )

	// pass to container
	self.Container:SetMediaInfo( tbl );

end


// define
derma.DefineControl( "MXMediaInfo", "", PANEL, "DPanel" );


// panel object
PANEL = {};

// custom skin hooks
Derma_Hook( PANEL, "ApplySchemeSettings", "Scheme", "MXMediaInfo" );


/*------------------------------------
	initialize the media info
------------------------------------*/
function PANEL:Init( )

	// create the label
	self.Label = vgui.Create( "MXLabel", self );
	// default position
	self.Label:SetPos( 0, self:GetTall() + 5 );
	
	// defaults
	self.AnimTime = 0;
	self.State = -1;
	self.CurrentText = "";

end


/*------------------------------------
	media info thinking
------------------------------------*/
function PANEL:Think( )

	// check if its time to animation
	if ( MXCurTime() > self.AnimTime && self.State != -1 ) then
	
		// set the next animation time
		self.AnimTime = MXCurTime() + 0.01;
		
		// default positions
		local x = self.Label.X;
		local y = self.Label.Y;
		
		// calculate the speed
		local speed = 1.4; /*( FrameTime() * 100 )*/
		
		// check if animation is vertical
		if ( self.State == 0 ) then
		
			// alter the Y position
			y = math.max( y - speed, 0 );
			
			// check if we've hit the top
			if ( y == 0 ) then
			
				// change the animation to horizontal
				self.State = 1;
				
				// delay the next animation
				self.AnimTime = MXCurTime() + 1;
				
			end
		
		// check if animation is horizontal
		elseif ( self.State == 1 ) then
		
			// minimum X position
			local min = ( self.Label:GetWide() + 10 ) * -1;
			
			// alter the X position
			x = math.max( x - speed, min );
			
			// check if we've hit the min position
			if ( x == min ) then
			
				// change the animation to reset
				self.State = 2;
				
				// delay the next animation sequence
				self.AnimTime = MXCurTime() + 1;
				
			end
			
		// check if we should reset
		elseif ( self.State == 2 ) then
		
			// reset position
			x = 5;
			y = self:GetTall() + 5;
			
			// reset animation state
			self.State = 0;
			
			// get the next media info text
			self:NextInfo();
		
		end
		
		// update the position
		self.Label:SetPos( x, y );
		
	end
	
end


/*------------------------------------
	paint the media info container
------------------------------------*/
function PANEL:Paint( )

	// check for special handling for $mediatime variable
	if ( self.CurrentText:find( "$mediatime" ) ) then
	
		// get the total duration
		local duration = MuseX.Player:GetDuration();
	
		// get the current position
		local pos = MuseX.Player:GetCurrentPosition();
		
		// build the time string
		local text = string.FormattedTime( pos, "%02i:%02i") .. " / " .. string.FormattedTime( duration, "%02i:%02i");
		
		// update the text
		self:SetText( self.CurrentText:gsub( "$mediatime", text ) );
	
	end
	
	// override
	return true;

end


/* CUSTOM PANEL FUNCTIONS */


/*------------------------------------
	set the media info table
------------------------------------*/
function PANEL:SetMediaInfo( tbl )

	// reste the label position
	self.Label:SetPos( 5, self:GetTall() + 5 );
	
	// store the table
	self.MediaInfoTable = tbl;
	
	// reset the media into position
	self.CurrentPos = 0;
	
	// get the next media info text
	self:NextInfo();
	
	// reset the animation state
	self.State = 0;
	
	// delay the next animation
	self.AnimTime = MXCurTime() + 1;

end


/*------------------------------------
	increment the media info
	position
------------------------------------*/
function PANEL:NextInfo( )

	// increment the position index
	self.CurrentPos = ( self.CurrentPos + 1 <= #self.MediaInfoTable ) && self.CurrentPos + 1 || 1;
	
	// update the text
	self.CurrentText = self.MediaInfoTable[ self.CurrentPos ] or "";
	
	// set the text
	self:SetText( self.CurrentText );

end


/*------------------------------------
	set the current media info text
------------------------------------*/
function PANEL:SetText( text )

	// update the label
	self.Label:SetText( text );
	
	// resize the label to contents
	self.Label:SizeToContents();

end

// define
derma.DefineControl( "MXMediaInfo_Container", "", PANEL, "DPanel" );
